@@ -22,11 +22,12 @@ from typing import TYPE_CHECKING, Annotated, cast
 from urllib.parse import ParseResult, unquote, urljoin, urlparse
 
 from fastapi import Depends, HTTPException, Request, status
-from fastapi.security import HTTPBearer, OAuth2PasswordBearer
+from fastapi.security import HTTPAuthorizationCredentials, HTTPBearer, OAuth2PasswordBearer
 from jwt import ExpiredSignatureError, InvalidTokenError
 from pydantic import NonNegativeInt
 
 from airflow.api_fastapi.app import get_auth_manager
+from airflow.api_fastapi.auth.managers.base_auth_manager import COOKIE_NAME_JWT_TOKEN
 from airflow.api_fastapi.auth.managers.models.base_user import BaseUser
 from airflow.api_fastapi.auth.managers.models.batch_apis import (
     IsAuthorizedConnectionRequest,
@@ -46,7 +47,14 @@ from airflow.api_fastapi.auth.managers.models.resource_details import (
     VariableDetails,
 )
 from airflow.api_fastapi.core_api.base import OrmClause
-from airflow.api_fastapi.core_api.datamodels.common import BulkAction, BulkBody
+from airflow.api_fastapi.core_api.datamodels.common import (
+    BulkAction,
+    BulkActionOnExistence,
+    BulkBody,
+    BulkCreateAction,
+    BulkDeleteAction,
+    BulkUpdateAction,
+)
 from airflow.api_fastapi.core_api.datamodels.connections import ConnectionBody
 from airflow.api_fastapi.core_api.datamodels.pools import PoolBody
 from airflow.api_fastapi.core_api.datamodels.variables import VariableBody
@@ -58,7 +66,6 @@ from airflow.models.taskinstance import TaskInstance as TI
 from airflow.models.xcom import XComModel
 
 if TYPE_CHECKING:
-    from fastapi.security import HTTPAuthorizationCredentials
     from sqlalchemy.sql import Select
 
     from airflow.api_fastapi.auth.managers.base_auth_manager import BaseAuthManager, ResourceMethod
@@ -96,14 +103,22 @@ async def resolve_user_from_token(token_str: str | None) -> BaseUser:
 
 
 async def get_user(
+    request: Request,
     oauth_token: str | None = Depends(oauth2_scheme),
     bearer_credentials: HTTPAuthorizationCredentials | None = Depends(bearer_scheme),
 ) -> BaseUser:
-    token_str = None
+    # A user might have been already built by a middleware, if so, it is stored in `request.state.user`
+    user: BaseUser | None = getattr(request.state, "user", None)
+    if user:
+        return user
+
+    token_str: str | None
     if bearer_credentials and bearer_credentials.scheme.lower() == "bearer":
         token_str = bearer_credentials.credentials
     elif oauth_token:
         token_str = oauth_token
+    else:
+        token_str = request.cookies.get(COOKIE_NAME_JWT_TOKEN)
 
     return await resolve_user_from_token(token_str)
 
@@ -257,19 +272,22 @@ def requires_access_pool_bulk() -> Callable[[BulkBody[PoolBody], BaseUser], None
     ) -> None:
         requests: list[IsAuthorizedPoolRequest] = []
         for action in request.actions:
-            requests.extend(
-                [
-                    {
-                        "method": MAP_BULK_ACTION_TO_AUTH_METHOD[action.action],
+            methods = _get_resource_methods_from_bulk_request(action)
+            for pool in action.entities:
+                pool_name = (
+                    cast("str", pool) if action.action == BulkAction.DELETE else cast("PoolBody", pool).pool
+                )
+                # For each pool, build a `IsAuthorizedPoolRequest`
+                # The list of `IsAuthorizedPoolRequest` will then be sent using `batch_is_authorized_pool`
+                # Each `IsAuthorizedPoolRequest` is similar to calling `is_authorized_pool`
+                for method in methods:
+                    req: IsAuthorizedPoolRequest = {
+                        "method": method,
                         "details": PoolDetails(
-                            name=cast("str", pool)
-                            if action.action == BulkAction.DELETE
-                            else cast("PoolBody", pool).pool
+                            name=pool_name,
                         ),
                     }
-                    for pool in action.entities
-                ]
-            )
+                    requests.append(req)
 
         _requires_access(
             is_authorized_callback=lambda: get_auth_manager().batch_is_authorized_pool(
@@ -307,19 +325,24 @@ def requires_access_connection_bulk() -> Callable[[BulkBody[ConnectionBody], Bas
     ) -> None:
         requests: list[IsAuthorizedConnectionRequest] = []
         for action in request.actions:
-            requests.extend(
-                [
-                    {
-                        "method": MAP_BULK_ACTION_TO_AUTH_METHOD[action.action],
+            methods = _get_resource_methods_from_bulk_request(action)
+            for connection in action.entities:
+                connection_id = (
+                    cast("str", connection)
+                    if action.action == BulkAction.DELETE
+                    else cast("ConnectionBody", connection).connection_id
+                )
+                # For each pool, build a `IsAuthorizedConnectionRequest`
+                # The list of `IsAuthorizedConnectionRequest` will then be sent using `batch_is_authorized_connection`
+                # Each `IsAuthorizedConnectionRequest` is similar to calling `is_authorized_connection`
+                for method in methods:
+                    req: IsAuthorizedConnectionRequest = {
+                        "method": method,
                         "details": ConnectionDetails(
-                            conn_id=cast("str", connection)
-                            if action.action == BulkAction.DELETE
-                            else cast("ConnectionBody", connection).connection_id
+                            conn_id=connection_id,
                         ),
                     }
-                    for connection in action.entities
-                ]
-            )
+                    requests.append(req)
 
         _requires_access(
             is_authorized_callback=lambda: get_auth_manager().batch_is_authorized_connection(
@@ -373,19 +396,24 @@ def requires_access_variable_bulk() -> Callable[[BulkBody[VariableBody], BaseUse
     ) -> None:
         requests: list[IsAuthorizedVariableRequest] = []
         for action in request.actions:
-            requests.extend(
-                [
-                    {
-                        "method": MAP_BULK_ACTION_TO_AUTH_METHOD[action.action],
+            methods = _get_resource_methods_from_bulk_request(action)
+            for variable in action.entities:
+                variable_key = (
+                    cast("str", variable)
+                    if action.action == BulkAction.DELETE
+                    else cast("VariableBody", variable).key
+                )
+                # For each variable, build a `IsAuthorizedVariableRequest`
+                # The list of `IsAuthorizedVariableRequest` will then be sent using `batch_is_authorized_variable`
+                # Each `IsAuthorizedVariableRequest` is similar to calling `is_authorized_variable`
+                for method in methods:
+                    req: IsAuthorizedVariableRequest = {
+                        "method": method,
                         "details": VariableDetails(
-                            key=cast("str", entity)
-                            if action.action == BulkAction.DELETE
-                            else cast("VariableBody", entity).key
+                            key=variable_key,
                         ),
                     }
-                    for entity in action.entities
-                ]
-            )
+                    requests.append(req)
 
         _requires_access(
             is_authorized_callback=lambda: get_auth_manager().batch_is_authorized_variable(
@@ -494,3 +522,15 @@ def is_safe_url(target_url: str, request: Request | None = None) -> bool:
         if parsed_target.scheme in {"http", "https"} and parsed_target.netloc == parsed_base.netloc:
             return True
     return False
+
+
+def _get_resource_methods_from_bulk_request(
+    action: BulkCreateAction | BulkUpdateAction | BulkDeleteAction,
+) -> list[ResourceMethod]:
+    resource_methods: list[ResourceMethod] = [MAP_BULK_ACTION_TO_AUTH_METHOD[action.action]]
+    # If ``action_on_existence`` == ``overwrite``, we need to check the user has ``PUT`` access as well.
+    # With ``action_on_existence`` == ``overwrite``, a create request is actually an update request if the
+    # resource already exists, hence adding this check.
+    if action.action == BulkAction.CREATE and action.action_on_existence == BulkActionOnExistence.OVERWRITE:
+        resource_methods.append("PUT")
+    return resource_methods
